@@ -1,7 +1,8 @@
 #
 #  Copyright (c) 1995-2001, Raphael Manfredi
-#  Copyright (c) 2002-2013 by the Perl 5 Porters
-#  
+#  Copyright (c) 2002-2014 by the Perl 5 Porters
+#  Copyright (c) 2015-2016 cPanel Inc
+#
 #  You may redistribute only under the same terms as Perl 5, as specified
 #  in the README file that comes with the distribution.
 #
@@ -18,14 +19,23 @@ package Storable; @ISA = qw(Exporter);
 	retrieve_fd
 	lock_store lock_nstore lock_retrieve
         file_magic read_magic
+	BLESS_OK TIE_OK FLAGS_COMPAT
 );
 
-use vars qw($canonical $forgive_me $VERSION);
+use vars qw($canonical $forgive_me $VERSION $XS_VERSION);
 
-$VERSION = '2.45';
+$VERSION = '3.05_01';
+$XS_VERSION = $VERSION;
+$VERSION = eval $VERSION;
 
 BEGIN {
-    if (eval { local $SIG{__DIE__}; require Log::Agent; 1 }) {
+    if (eval {
+        local $SIG{__DIE__};
+        local @INC = @INC;
+        pop @INC if $INC[-1] eq '.';
+        require Log::Agent;
+        1;
+    }) {
         Log::Agent->import;
     }
     #
@@ -51,14 +61,14 @@ BEGIN {
 #
 
 BEGIN {
-	if (eval { require Fcntl; 1 } && exists $Fcntl::EXPORT_TAGS{'flock'}) {
-		Fcntl->import(':flock');
-	} else {
-		eval q{
-			sub LOCK_SH ()	{1}
-			sub LOCK_EX ()	{2}
-		};
-	}
+    if (eval { require Fcntl; 1 } && exists $Fcntl::EXPORT_TAGS{'flock'}) {
+        Fcntl->import(':flock');
+    } else {
+        eval q{
+	          sub LOCK_SH () { 1 }
+		  sub LOCK_EX () { 2 }
+	      };
+    }
 }
 
 sub CLONE {
@@ -66,25 +76,31 @@ sub CLONE {
     Storable::init_perinterp();
 }
 
+sub BLESS_OK     () { 2 }
+sub TIE_OK       () { 4 }
+sub FLAGS_COMPAT () { BLESS_OK | TIE_OK }
+
 # By default restricted hashes are downgraded on earlier perls.
 
+$Storable::flags = FLAGS_COMPAT;
 $Storable::downgrade_restricted = 1;
 $Storable::accept_future_minor = 1;
 
-XSLoader::load('Storable', $Storable::VERSION);
+XSLoader::load('Storable');
 
 #
 # Determine whether locking is possible, but only when needed.
 #
 
-sub CAN_FLOCK; my $CAN_FLOCK; sub CAN_FLOCK {
-	return $CAN_FLOCK if defined $CAN_FLOCK;
-	require Config; import Config;
-	return $CAN_FLOCK =
-		$Config{'d_flock'} ||
-		$Config{'d_fcntl_can_lock'} ||
-		$Config{'d_lockf'};
+my $CAN_FLOCK;
+BEGIN {
+    require Config;
+    $CAN_FLOCK =
+      $Config::Config{'d_flock'} ||
+      $Config::Config{'d_fcntl_can_lock'} ||
+      $Config::Config{'d_lockf'};
 }
+sub CAN_FLOCK () { $CAN_FLOCK } 
 
 sub show_file_magic {
     print <<EOM;
@@ -113,7 +129,7 @@ sub file_magic {
 
     my $file = shift;
     my $fh = IO::File->new;
-    open($fh, "<". $file) || die "Can't open '$file': $!";
+    open($fh, "<", $file) || die "Can't open '$file': $!";
     binmode($fh);
     defined(sysread($fh, my $buf, 32)) || die "Can't read from '$file': $!";
     close($fh);
@@ -200,7 +216,7 @@ sub BIN_WRITE_VERSION_NV {
 # removed.
 #
 sub store {
-	return _store(\&pstore, @_, 0);
+    return _store(\&pstore, @_, 0);
 }
 
 #
@@ -209,7 +225,7 @@ sub store {
 # Same as store, but in network order.
 #
 sub nstore {
-	return _store(\&net_pstore, @_, 0);
+    return _store(\&net_pstore, @_, 0);
 }
 
 #
@@ -218,7 +234,7 @@ sub nstore {
 # Same as store, but flock the file first (advisory locking).
 #
 sub lock_store {
-	return _store(\&pstore, @_, 1);
+    return _store(\&pstore, @_, 1);
 }
 
 #
@@ -227,47 +243,48 @@ sub lock_store {
 # Same as nstore, but flock the file first (advisory locking).
 #
 sub lock_nstore {
-	return _store(\&net_pstore, @_, 1);
+    return _store(\&net_pstore, @_, 1);
 }
 
 # Internal store to file routine
 sub _store {
-	my $xsptr = shift;
-	my $self = shift;
-	my ($file, $use_locking) = @_;
-	logcroak "not a reference" unless ref($self);
-	logcroak "wrong argument number" unless @_ == 2;	# No @foo in arglist
-	local *FILE;
-	if ($use_locking) {
-		open(FILE, ">>$file") || logcroak "can't write into $file: $!";
-		unless (&CAN_FLOCK) {
-			logcarp "Storable::lock_store: fcntl/flock emulation broken on $^O";
-			return undef;
-		}
-		flock(FILE, LOCK_EX) ||
-			logcroak "can't get exclusive lock on $file: $!";
-		truncate FILE, 0;
-		# Unlocking will happen when FILE is closed
-	} else {
-		open(FILE, ">$file") || logcroak "can't create $file: $!";
-	}
-	binmode FILE;				# Archaic systems...
-	my $da = $@;				# Don't mess if called from exception handler
-	my $ret;
-	# Call C routine nstore or pstore, depending on network order
-	eval { $ret = &$xsptr(*FILE, $self) };
-	# close will return true on success, so the or short-circuits, the ()
-	# expression is true, and for that case the block will only be entered
-	# if $@ is true (ie eval failed)
-	# if close fails, it returns false, $ret is altered, *that* is (also)
-	# false, so the () expression is false, !() is true, and the block is
-	# entered.
-	if (!(close(FILE) or undef $ret) || $@) {
-		unlink($file) or warn "Can't unlink $file: $!\n";
-	}
-	logcroak $@ if $@ =~ s/\.?\n$/,/;
-	$@ = $da;
-	return $ret;
+    my $xsptr = shift;
+    my $self = shift;
+    my ($file, $use_locking) = @_;
+    logcroak "not a reference" unless ref($self);
+    logcroak "wrong argument number" unless @_ == 2;	# No @foo in arglist
+    local *FILE;
+    if ($use_locking) {
+        open(FILE, ">>", $file) || logcroak "can't write into $file: $!";
+        unless (&CAN_FLOCK) {
+            logcarp
+              "Storable::lock_store: fcntl/flock emulation broken on $^O";
+            return undef;
+        }
+        flock(FILE, LOCK_EX) ||
+          logcroak "can't get exclusive lock on $file: $!";
+        truncate FILE, 0;
+        # Unlocking will happen when FILE is closed
+    } else {
+        open(FILE, ">", $file) || logcroak "can't create $file: $!";
+    }
+    binmode FILE;	# Archaic systems...
+    my $da = $@;	# Don't mess if called from exception handler
+    my $ret;
+    # Call C routine nstore or pstore, depending on network order
+    eval { $ret = &$xsptr(*FILE, $self) };
+    # close will return true on success, so the or short-circuits, the ()
+    # expression is true, and for that case the block will only be entered
+    # if $@ is true (ie eval failed)
+    # if close fails, it returns false, $ret is altered, *that* is (also)
+    # false, so the () expression is false, !() is true, and the block is
+    # entered.
+    if (!(close(FILE) or undef $ret) || $@) {
+        unlink($file) or warn "Can't unlink $file: $!\n";
+    }
+    logcroak $@ if $@ =~ s/\.?\n$/,/;
+    $@ = $da;
+    return $ret;
 }
 
 #
@@ -277,7 +294,7 @@ sub _store {
 # Returns undef if an I/O error occurred.
 #
 sub store_fd {
-	return _store_fd(\&pstore, @_);
+    return _store_fd(\&pstore, @_);
 }
 
 #
@@ -286,27 +303,27 @@ sub store_fd {
 # Same as store_fd, but in network order.
 #
 sub nstore_fd {
-	my ($self, $file) = @_;
-	return _store_fd(\&net_pstore, @_);
+    my ($self, $file) = @_;
+    return _store_fd(\&net_pstore, @_);
 }
 
 # Internal store routine on opened file descriptor
 sub _store_fd {
-	my $xsptr = shift;
-	my $self = shift;
-	my ($file) = @_;
-	logcroak "not a reference" unless ref($self);
-	logcroak "too many arguments" unless @_ == 1;	# No @foo in arglist
-	my $fd = fileno($file);
-	logcroak "not a valid file descriptor" unless defined $fd;
-	my $da = $@;				# Don't mess if called from exception handler
-	my $ret;
-	# Call C routine nstore or pstore, depending on network order
-	eval { $ret = &$xsptr($file, $self) };
-	logcroak $@ if $@ =~ s/\.?\n$/,/;
-	local $\; print $file '';	# Autoflush the file if wanted
-	$@ = $da;
-	return $ret;
+    my $xsptr = shift;
+    my $self = shift;
+    my ($file) = @_;
+    logcroak "not a reference" unless ref($self);
+    logcroak "too many arguments" unless @_ == 1;	# No @foo in arglist
+    my $fd = fileno($file);
+    logcroak "not a valid file descriptor" unless defined $fd;
+    my $da = $@;		# Don't mess if called from exception handler
+    my $ret;
+    # Call C routine nstore or pstore, depending on network order
+    eval { $ret = &$xsptr($file, $self) };
+    logcroak $@ if $@ =~ s/\.?\n$/,/;
+    local $\; print $file '';	# Autoflush the file if wanted
+    $@ = $da;
+    return $ret;
 }
 
 #
@@ -316,7 +333,7 @@ sub _store_fd {
 # containing the result.
 #
 sub freeze {
-	_freeze(\&mstore, @_);
+    _freeze(\&mstore, @_);
 }
 
 #
@@ -325,22 +342,22 @@ sub freeze {
 # Same as freeze but in network order.
 #
 sub nfreeze {
-	_freeze(\&net_mstore, @_);
+    _freeze(\&net_mstore, @_);
 }
 
 # Internal freeze routine
 sub _freeze {
-	my $xsptr = shift;
-	my $self = shift;
-	logcroak "not a reference" unless ref($self);
-	logcroak "too many arguments" unless @_ == 0;	# No @foo in arglist
-	my $da = $@;				# Don't mess if called from exception handler
-	my $ret;
-	# Call C routine mstore or net_mstore, depending on network order
-	eval { $ret = &$xsptr($self) };
-	logcroak $@ if $@ =~ s/\.?\n$/,/;
-	$@ = $da;
-	return $ret ? $ret : undef;
+    my $xsptr = shift;
+    my $self = shift;
+    logcroak "not a reference" unless ref($self);
+    logcroak "too many arguments" unless @_ == 0;	# No @foo in arglist
+    my $da = $@;	        # Don't mess if called from exception handler
+    my $ret;
+    # Call C routine mstore or net_mstore, depending on network order
+    eval { $ret = &$xsptr($self) };
+    logcroak $@ if $@ =~ s/\.?\n$/,/;
+    $@ = $da;
+    return $ret ? $ret : undef;
 }
 
 #
@@ -349,8 +366,13 @@ sub _freeze {
 # Retrieve object hierarchy from disk, returning a reference to the root
 # object of that tree.
 #
+# retrieve(file, flags)
+# flags include by default BLESS_OK=2 | TIE_OK=4
+# with flags=0 or the global $Storable::flags set to 0, no resulting object
+# will be blessed nor tied.
+#
 sub retrieve {
-	_retrieve($_[0], 0);
+    _retrieve(shift, 0, @_);
 }
 
 #
@@ -359,30 +381,32 @@ sub retrieve {
 # Same as retrieve, but with advisory locking.
 #
 sub lock_retrieve {
-	_retrieve($_[0], 1);
+    _retrieve(shift, 1, @_);
 }
 
 # Internal retrieve routine
 sub _retrieve {
-	my ($file, $use_locking) = @_;
-	local *FILE;
-	open(FILE, $file) || logcroak "can't open $file: $!";
-	binmode FILE;							# Archaic systems...
-	my $self;
-	my $da = $@;							# Could be from exception handler
-	if ($use_locking) {
-		unless (&CAN_FLOCK) {
-			logcarp "Storable::lock_store: fcntl/flock emulation broken on $^O";
-			return undef;
-		}
-		flock(FILE, LOCK_SH) || logcroak "can't get shared lock on $file: $!";
-		# Unlocking will happen when FILE is closed
-	}
-	eval { $self = pretrieve(*FILE) };		# Call C routine
-	close(FILE);
-	logcroak $@ if $@ =~ s/\.?\n$/,/;
-	$@ = $da;
-	return $self;
+    my ($file, $use_locking, $flags) = @_;
+    $flags = $Storable::flags unless defined $flags;
+    my $FILE;
+    open($FILE, "<", $file) || logcroak "can't open $file: $!";
+    binmode $FILE;			# Archaic systems...
+    my $self;
+    my $da = $@;			# Could be from exception handler
+    if ($use_locking) {
+        unless (&CAN_FLOCK) {
+            logcarp
+              "Storable::lock_store: fcntl/flock emulation broken on $^O";
+            return undef;
+        }
+        flock($FILE, LOCK_SH) || logcroak "can't get shared lock on $file: $!";
+        # Unlocking will happen when FILE is closed
+    }
+    eval { $self = pretrieve($FILE, $flags) };		# Call C routine
+    close($FILE);
+    logcroak $@ if $@ =~ s/\.?\n$/,/;
+    $@ = $da;
+    return $self;
 }
 
 #
@@ -391,15 +415,16 @@ sub _retrieve {
 # Same as retrieve, but perform from an already opened file descriptor instead.
 #
 sub fd_retrieve {
-	my ($file) = @_;
-	my $fd = fileno($file);
-	logcroak "not a valid file descriptor" unless defined $fd;
-	my $self;
-	my $da = $@;							# Could be from exception handler
-	eval { $self = pretrieve($file) };		# Call C routine
-	logcroak $@ if $@ =~ s/\.?\n$/,/;
-	$@ = $da;
-	return $self;
+    my ($file, $flags) = @_;
+    $flags = $Storable::flags unless defined $flags;
+    my $fd = fileno($file);
+    logcroak "not a valid file descriptor" unless defined $fd;
+    my $self;
+    my $da = $@;				# Could be from exception handler
+    eval { $self = pretrieve($file, $flags) };	# Call C routine
+    logcroak $@ if $@ =~ s/\.?\n$/,/;
+    $@ = $da;
+    return $self;
 }
 
 sub retrieve_fd { &fd_retrieve }		# Backward compatibility
@@ -410,15 +435,21 @@ sub retrieve_fd { &fd_retrieve }		# Backward compatibility
 # Recreate objects in memory from an existing frozen image created
 # by freeze.  If the frozen image passed is undef, return undef.
 #
+# thaw(frozen_obj, flags)
+# flags include by default BLESS_OK=2 | TIE_OK=4
+# with flags=0 or the global $Storable::flags set to 0, no resulting object
+# will be blessed nor tied.
+#
 sub thaw {
-	my ($frozen) = @_;
-	return undef unless defined $frozen;
-	my $self;
-	my $da = $@;							# Could be from exception handler
-	eval { $self = mretrieve($frozen) };	# Call C routine
-	logcroak $@ if $@ =~ s/\.?\n$/,/;
-	$@ = $da;
-	return $self;
+    my ($frozen, $flags) = @_;
+    $flags = $Storable::flags unless defined $flags;
+    return undef unless defined $frozen;
+    my $self;
+    my $da = $@;			        # Could be from exception handler
+    eval { $self = mretrieve($frozen, $flags) };# Call C routine
+    logcroak $@ if $@ =~ s/\.?\n$/,/;
+    $@ = $da;
+    return $self;
 }
 
 1;
@@ -1025,31 +1056,33 @@ compartment:
 B<Do not accept Storable documents from untrusted sources!>
 
 Some features of Storable can lead to security vulnerabilities if you
-accept Storable documents from untrusted sources. Most obviously, the
-optional (off by default) CODE reference serialization feature allows
-transfer of code to the deserializing process. Furthermore, any
-serialized object will cause Storable to helpfully load the module
-corresponding to the class of the object in the deserializing module.
-For manipulated module names, this can load almost arbitrary code.
-Finally, the deserialized object's destructors will be invoked when
-the objects get destroyed in the deserializing process. Maliciously
-crafted Storable documents may put such objects in the value of
-a hash key that is overridden by another key/value pair in the
-same hash, thus causing immediate destructor execution.
+accept Storable documents from untrusted sources with the default
+flags. Most obviously, the optional (off by default) CODE reference
+serialization feature allows transfer of code to the deserializing
+process. Furthermore, any serialized object will cause Storable to
+helpfully load the module corresponding to the class of the object in
+the deserializing module.  For manipulated module names, this can load
+almost arbitrary code.  Finally, the deserialized object's destructors
+will be invoked when the objects get destroyed in the deserializing
+process. Maliciously crafted Storable documents may put such objects
+in the value of a hash key that is overridden by another key/value
+pair in the same hash, thus causing immediate destructor execution.
 
-In a future version of Storable, we intend to provide options to disable
-loading modules for classes and to disable deserializing objects
-altogether. I<Nonetheless, Storable deserializing documents from
-untrusted sources is expected to have other, yet undiscovered,
-security concerns such as allowing an attacker to cause the deserializer
-to crash hard.>
+To disable blessing objects while thawing/retrieving remove the flag
+BLESS_OK = 2 from C<$Storable::flags> or set the 2nd argument for
+thaw/retrieve to 0.
 
-B<Therefore, let me repeat: Do not accept Storable documents from
-untrusted sources!>
+To disable tieing data while thawing/retrieving remove the flag TIE_OK
+= 4 from C<$Storable::flags> or set the 2nd argument for thaw/retrieve
+to 0.
+
+With the default setting of $Storable::flags = 6, creating or destroying
+random objects, even renamed objects can be controlled by an attacker.
+See CVE-2015-1592 and its metasploit module.
 
 If your application requires accepting data from untrusted sources, you
 are best off with a less powerful and more-likely safe serialization format
-and implementation. If your data is sufficiently simple, JSON is a good
+and implementation. If your data is sufficiently simple, JSON is the best
 choice and offers maximum interoperability.
 
 =head1 WARNING
@@ -1086,8 +1119,8 @@ deal with them.
 
 The store functions will C<croak> if they run into such references
 unless you set C<$Storable::forgive_me> to some C<TRUE> value. In that
-case, the fatal message is turned in a warning and some
-meaningless string is stored instead.
+case, the fatal message is converted to a warning and some meaningless
+string is stored instead.
 
 Setting C<$Storable::canonical> may not yield frozen strings that
 compare equal due to possible stringification of numbers. When the
@@ -1121,6 +1154,34 @@ to string or floating point.  In other words values that had been generated
 by integer operations such as logic ops and then not used in any string or
 arithmetic context before storing.
 
+=head2 Large data on 64-bit platforms
+
+Storable's current data format is incapable of representing lengths greater
+than fit into a signed 32-bit integer, or about 2 GB. In practice, this
+means that, even with the latest Perl and a 64-bit machine with plenty of
+memory, you cannot store, retrieve, or clone any of the following:
+
+=over 4
+
+=item *
+
+A string containing 2**31 or more bytes (including as an element of an array, or
+a key or value in a hash)
+
+=item *
+
+An array with 2**31 or more elements
+
+=item *
+
+A hash with 2**31 or more keys
+
+=back
+
+Attempting to do so will yield an exception.
+
+This may be fixed in the future.
+
 =head2 64 bit data in perl 5.6.0 and 5.6.1
 
 This section only applies to you if you have existing data written out
@@ -1195,6 +1256,8 @@ Thank you to (in chronological order):
 	Erik Haugan <erik@solbors.no>
 	Benjamin A. Holzman <ben.holzman@grantstreet.com>
 	Reini Urban <rurban@cpanel.net>
+	Todd Rinaldo <toddr@cpanel.net>
+	Aaron Crane <arc@cpan.org>
 
 for their bug reports, suggestions and contributions.
 
@@ -1212,8 +1275,10 @@ the bill.
 
 =head1 AUTHOR
 
-Storable was written by Raphael Manfredi F<E<lt>Raphael_Manfredi@pobox.comE<gt>>
-Maintenance is now done by the perl5-porters F<E<lt>perl5-porters@perl.orgE<gt>>
+Storable was written by Raphael Manfredi
+F<E<lt>Raphael_Manfredi@pobox.comE<gt>>
+Maintenance is now done by the perl5-porters
+F<E<lt>perl5-porters@perl.orgE<gt>>
 
 Please e-mail us with problems, bug fixes, comments and complaints,
 although if you have compliments you should send them to Raphael.
